/**
 * Go SDK for OpenFGA
 *
 * API version: 1.x
 * Website: https://openfga.dev
 * Documentation: https://openfga.dev/docs
 * Support: https://openfga.dev/community
 * License: [Apache-2.0](https://github.com/openfga/go-sdk/blob/main/LICENSE)
 *
 * NOTE: This file was auto generated by OpenAPI Generator (https://openapi-generator.tech). DO NOT EDIT.
 */

package openfga

import (
	"bytes"

	"encoding/json"
)

// WriteAuthorizationModelResponse struct for WriteAuthorizationModelResponse
type WriteAuthorizationModelResponse struct {
	AuthorizationModelId string `json:"authorization_model_id" yaml:"authorization_model_id"`
}

// NewWriteAuthorizationModelResponse instantiates a new WriteAuthorizationModelResponse object
// This constructor will assign default values to properties that have it defined,
// and makes sure properties required by API are set, but the set of arguments
// will change when the set of required properties is changed
func NewWriteAuthorizationModelResponse(authorizationModelId string) *WriteAuthorizationModelResponse {
	this := WriteAuthorizationModelResponse{}
	this.AuthorizationModelId = authorizationModelId
	return &this
}

// NewWriteAuthorizationModelResponseWithDefaults instantiates a new WriteAuthorizationModelResponse object
// This constructor will only assign default values to properties that have it defined,
// but it doesn't guarantee that properties required by API are set
func NewWriteAuthorizationModelResponseWithDefaults() *WriteAuthorizationModelResponse {
	this := WriteAuthorizationModelResponse{}
	return &this
}

// GetAuthorizationModelId returns the AuthorizationModelId field value
func (o *WriteAuthorizationModelResponse) GetAuthorizationModelId() string {
	if o == nil {
		var ret string
		return ret
	}

	return o.AuthorizationModelId
}

// GetAuthorizationModelIdOk returns a tuple with the AuthorizationModelId field value
// and a boolean to check if the value has been set.
func (o *WriteAuthorizationModelResponse) GetAuthorizationModelIdOk() (*string, bool) {
	if o == nil {
		return nil, false
	}
	return &o.AuthorizationModelId, true
}

// SetAuthorizationModelId sets field value
func (o *WriteAuthorizationModelResponse) SetAuthorizationModelId(v string) {
	o.AuthorizationModelId = v
}

func (o WriteAuthorizationModelResponse) MarshalJSON() ([]byte, error) {
	toSerialize := map[string]interface{}{}
	toSerialize["authorization_model_id"] = o.AuthorizationModelId
	var b bytes.Buffer
	enc := json.NewEncoder(&b)
	enc.SetEscapeHTML(false)
	err := enc.Encode(toSerialize)
	if err != nil {
		return nil, err
	}
	return b.Bytes(), nil
}

type NullableWriteAuthorizationModelResponse struct {
	value *WriteAuthorizationModelResponse
	isSet bool
}

func (v NullableWriteAuthorizationModelResponse) Get() *WriteAuthorizationModelResponse {
	return v.value
}

func (v *NullableWriteAuthorizationModelResponse) Set(val *WriteAuthorizationModelResponse) {
	v.value = val
	v.isSet = true
}

func (v NullableWriteAuthorizationModelResponse) IsSet() bool {
	return v.isSet
}

func (v *NullableWriteAuthorizationModelResponse) Unset() {
	v.value = nil
	v.isSet = false
}

func NewNullableWriteAuthorizationModelResponse(val *WriteAuthorizationModelResponse) *NullableWriteAuthorizationModelResponse {
	return &NullableWriteAuthorizationModelResponse{value: val, isSet: true}
}

func (v NullableWriteAuthorizationModelResponse) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.value)
}

func (v *NullableWriteAuthorizationModelResponse) UnmarshalJSON(src []byte) error {
	v.isSet = true
	return json.Unmarshal(src, &v.value)
}
